package de.hendriklipka.q3master;

import org.log4j.*;

import java.net.*;
import java.io.IOException;

public class BroadcastQuery extends Thread
{
    private static Category CAT=Category.getInstance(BroadcastQuery.class.getName());
    private static Category LOG_CAT=Category.getInstance("q3master.broadcast");

    protected String[] _addresses=null;
    protected int _sleepTime=0;

    protected StatusHandler _handler=null;

    protected DatagramSocket _socket=null;

    public BroadcastQuery(String[] addresses, int sleepTime, StatusHandler handler)
    {
        _addresses=addresses;
        _sleepTime=sleepTime;
        _handler=handler;
    }

    public void run()
    {
        _socket=de.hendriklipka.util.UDPSocketPool.getSocket();
        new BroadcastListener().start();
        while (true)
        {
            for (int i=0;i<_addresses.length;i++)
            {
                CAT.debug("sending broadcast packet to "+_addresses[i]);
                try
                {
                    Util.sendRequest(_addresses[i],"getstatus".getBytes(),_socket,27960);
                }
                catch (IOException ex)
                {
                    CAT.error(ex);
                }
            }
            Object sleepObj=new Object();
            synchronized (sleepObj)
            {
                try
                {
                    sleepObj.wait(_sleepTime);
                }
                catch (InterruptedException ex) {}
            }
        }
    }

    protected class BroadcastListener extends Thread
    {
        public void run()
        {
            while (true)
            {
                try
                {
                    byte[] buffer=new byte[65507]; // maximum packet size
                    DatagramPacket pr=new DatagramPacket(buffer,buffer.length);
                    _socket.receive(pr);
                    InetAddress from=pr.getAddress();
                    int fromPort=pr.getPort();
                    LOG_CAT.info("received broadcast answer from "+from+":"+fromPort);
                    ServerData sd=new ServerData();
                    sd._serverAddress=from;
                    sd._serverPort=fromPort;
                    sd._lastHeartBeat=System.currentTimeMillis();
                    _handler.addServerByBroadcast(sd);
                }
                catch (Exception ex)
                {
                    CAT.error(ex);
                }
            }
        }
    }
}